@@ -60,6 +60,11 @@ static gboolean              gail_button_set_description(AtkAction      *action,
 static void                  atk_image_interface_init   (AtkImageIface  *iface);
 static G_CONST_RETURN gchar* gail_button_get_image_description 
                                                         (AtkImage       *image);
+static void	             gail_button_get_image_position
+                                                        (AtkImage       *image,
+                                                         gint	        *x,
+                                                         gint	        *y,
+                                                         AtkCoordType   coord_type);
 static void                  gail_button_get_image_size (AtkImage       *image,
                                                          gint           *width,
                                                          gint           *height);
@@ -291,6 +296,9 @@ gail_button_do_action (AtkAction *action,
      */
     return FALSE;
 
+  if (!GTK_WIDGET_SENSITIVE (widget) || !GTK_WIDGET_VISIBLE (widget))
+    return FALSE;
+
   button = GTK_BUTTON (widget); 
   switch (i)
     {
@@ -565,6 +573,7 @@ atk_image_interface_init (AtkImageIface *iface)
   g_return_if_fail (iface != NULL);
 
   iface->get_image_description = gail_button_get_image_description;
+  iface->get_image_position = gail_button_get_image_position;
   iface->get_image_size = gail_button_get_image_size;
   iface->set_image_description = gail_button_set_image_description;
 }
@@ -579,9 +588,11 @@ get_image_from_button (GtkWidget *button)
   child = gtk_bin_get_child (GTK_BIN (button));
   if (GTK_IS_ALIGNMENT (child))
     child = gtk_bin_get_child (GTK_BIN (child));
-  if (GTK_IS_BOX (child))
+  if (GTK_IS_CONTAINER (child))
     {
-      list = gtk_container_children (GTK_CONTAINER (child));
+      list = gtk_container_get_children (GTK_CONTAINER (child));
+      if (!list)
+        return NULL;
       if (GTK_IS_IMAGE (list->data))
         image = GTK_IMAGE (list->data);
       g_list_free (list);
@@ -616,6 +627,42 @@ gail_button_get_image_description (AtkImage *image) {
     return NULL;
 }
 
+static void
+gail_button_get_image_position (AtkImage     *image,
+                                gint	     *x,
+                                gint	     *y,
+                                AtkCoordType coord_type)
+{
+  GtkWidget *widget;
+  GtkImage  *button_image;
+  AtkObject *obj;
+
+  widget = GTK_ACCESSIBLE (image)->widget;
+
+  if (widget == NULL)
+    {
+    /*
+     * State is defunct
+     */
+      *x = G_MININT;
+      *y = G_MININT;
+      return;
+    }
+
+  button_image = get_image_from_button (widget);
+
+  if (button_image != NULL)
+    {
+      obj = gtk_widget_get_accessible (GTK_WIDGET (button_image));
+      atk_component_get_position (ATK_COMPONENT (obj), x, y, coord_type); 
+    }
+  else
+    {
+      *x = G_MININT;
+      *y = G_MININT;
+    }
+}
+
 static void
 gail_button_get_image_size (AtkImage *image,
                             gint     *width,
@@ -628,12 +675,14 @@ gail_button_get_image_size (AtkImage *image,
   widget = GTK_ACCESSIBLE (image)->widget;
 
   if (widget == NULL)
+    {
     /*
      * State is defunct
      */
-    *width = -1;
-    *height = -1;
-    return;
+      *width = -1;
+      *height = -1;
+      return;
+    }
 
   button_image = get_image_from_button (widget);
 
@@ -696,17 +745,25 @@ get_label_from_button (GtkWidget *button)
   child = gtk_bin_get_child (GTK_BIN (button));
   if (GTK_IS_ALIGNMENT (child))
     child = gtk_bin_get_child (GTK_BIN (child));
-  if (GTK_IS_BOX (child))
+  if (GTK_IS_CONTAINER (child))
     {
       /*
-       * Child is not a label; perhaps the button is for a stock item.
-       * If it is, then the second child of the child should be a label.
+       * Child is not a label.  Search for a label in the container's children.
        */
       GList *children, *tmp_list;
  
-      children = gtk_container_children (GTK_CONTAINER (child));
-      tmp_list = g_list_nth (children, 1);
-      child = GTK_WIDGET (tmp_list->data);
+      children = gtk_container_get_children (GTK_CONTAINER (child));
+
+      child = NULL;
+      for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
+	{
+	  if (GTK_IS_LABEL (tmp_list->data))
+	    {
+	      child = GTK_WIDGET (tmp_list->data);
+	      break;
+	    }  
+	}
+
       g_list_free (children);
     }
   return child;
