@@ -343,7 +343,9 @@ gail_cell_remove_action (GailCell *cell,
 
   g_return_val_if_fail (GAIL_IS_CELL (cell), FALSE);
   list_node = g_list_nth (cell->action_list, action_index);
-  g_return_val_if_fail ((list_node != NULL), FALSE);
+  if (!list_node)
+    return FALSE;
+  g_return_val_if_fail (list_node->data != NULL, FALSE);
   _gail_cell_destroy_action_info (list_node->data, NULL);
   cell->action_list = g_list_remove_link (cell->action_list, list_node);
   return TRUE;
@@ -383,7 +385,8 @@ _gail_cell_get_action_info (GailCell *cell,
   if (cell->action_list == NULL)
     return NULL;
   list_node = g_list_nth (cell->action_list, index);
-  g_return_val_if_fail (list_node, NULL);
+  if (!list_node)
+    return NULL;
   return (ActionInfo *) (list_node->data);
 }
 
