@@ -651,7 +651,12 @@ gail_entry_get_offset_at_point (AtkText *text,
   index = gail_misc_get_index_at_point_in_layout (widget, 
                gtk_entry_get_layout(entry), x_layout, y_layout, x, y, coords);
   if (index == -1)
-    return -1;  
+    {
+      if (coords == ATK_XY_SCREEN || coords == ATK_XY_WINDOW)
+        return g_utf8_strlen (entry->text, -1);
+
+      return index;  
+    }
   else
     {
       cursor_index = g_utf8_offset_to_pointer (entry->text, 
@@ -1026,9 +1031,17 @@ gail_entry_get_character_at_offset (AtkText	         *text,
 
   entry = GTK_ENTRY (widget);
   string = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-  index = g_utf8_offset_to_pointer (string, offset);
+  if (offset >= g_utf8_strlen (string, -1))
+    {
+      unichar = '\0';
+    }
+  else
+    {
+      index = g_utf8_offset_to_pointer (string, offset);
+
+      unichar = g_utf8_get_char(index);
+    }
 
-  unichar = g_utf8_get_char(index);
   g_free(string);
   return unichar;
 }
