@@ -21,8 +21,7 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include "gailtextview.h"
-#include "gailtextviewfactory.h"
-#include <util/gailmisc.h>
+#include <libgail-util/gailmisc.h>
 
 static void       gail_text_view_class_init            (GailTextViewClass *klass);
 static void       gail_text_view_init                  (GailTextView      *text_view);
@@ -150,7 +149,8 @@ static gchar*            get_text_near_offset          (AtkText          *text,
                                                         gint             *start_offset,
                                                         gint             *end_offset);
 static gint             get_insert_offset              (GtkTextBuffer    *buffer);
-static void             emit_text_caret_moved          (AtkObject        *obj,
+static gint             get_selection_bound            (GtkTextBuffer    *buffer);
+static void             emit_text_caret_moved          (GailTextView     *gail_text_view,
                                                         gint             insert_offset);
 
 static GailWidgetClass *parent_class = NULL;
@@ -234,6 +234,7 @@ gail_text_view_init (GailTextView      *text_view)
   text_view->textutil = NULL;
   text_view->signal_name = NULL;
   text_view->previous_insert_offset = -1;
+  text_view->previous_selection_bound = -1;
 }
 
 static void
@@ -298,10 +299,12 @@ gail_text_view_real_notify_gtk (GObject             *obj,
   if (!strcmp (pspec->name, "editable"))
     {
       GValue value = { 0, };
+      AtkObject *atk_obj;
 
       g_value_init (&value, G_TYPE_BOOLEAN);
       g_object_get_property (obj, "editable", &value);
-      atk_object_notify_state_change (ATK_OBJECT (obj), ATK_STATE_EDITABLE,
+      atk_obj = gtk_widget_get_accessible (GTK_WIDGET (obj));
+      atk_object_notify_state_change (atk_obj, ATK_STATE_EDITABLE,
                                       g_value_get_boolean (&value));
     }
   else
@@ -447,19 +450,24 @@ gail_text_view_get_character_at_offset (AtkText *text,
   GtkTextBuffer *buffer;
   gchar *string;
   gchar *index;
+  gunichar unichar;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
     return '\0';
 
   buffer = GAIL_TEXT_VIEW (text)->textutil->buffer;
+  if (offset >= gtk_text_buffer_get_char_count (buffer))
+    return '\0';
+
   gtk_text_buffer_get_start_iter (buffer, &start);
   gtk_text_buffer_get_end_iter (buffer, &end);
   string = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
   index = g_utf8_offset_to_pointer (string, offset);
-  g_free(string);
 
-  return g_utf8_get_char (index);
+  unichar = g_utf8_get_char (index);
+  g_free(string);
+  return unichar;
 }
 
 static gint
@@ -528,6 +536,7 @@ gail_text_view_get_offset_at_point (AtkText      *text,
   gint x_widget, y_widget, x_window, y_window, buff_x, buff_y;
   GtkWidget *widget;
   GdkWindow *window;
+  GdkRectangle rect;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
@@ -558,6 +567,13 @@ gail_text_view_get_offset_at_point (AtkText      *text,
 
   gtk_text_view_window_to_buffer_coords (view, GTK_TEXT_WINDOW_WIDGET,
                                          x, y, &buff_x, &buff_y);
+  gtk_text_view_get_visible_rect (view, &rect);
+  /*
+   * Clamp point to visible rectangle
+   */
+  buff_x = CLAMP (buff_x, rect.x, rect.x + rect.width - 1);
+  buff_y = CLAMP (buff_y, rect.y, rect.y + rect.height - 1);
+
   gtk_text_view_get_iter_at_location (view, &loc_itr, buff_x, buff_y);
   return gtk_text_iter_get_offset (&loc_itr);
 }
@@ -970,10 +986,10 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
   GtkTextTag *tag;
   GtkTextIter start;
   GtkTextIter end;
-  gint len;
-  gint index, j;
+  gint j;
   GdkColor *color;
   gchar** RGB_vals;
+  GSList *l;
 
   widget = GTK_ACCESSIBLE (text)->widget;
   if (widget == NULL)
@@ -981,9 +997,12 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
     return FALSE;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return FALSE;
+
   buffer = view->buffer;
 
-  if (attrib_set == NULL || (len = g_slist_length (attrib_set)) == 0)
+  if (attrib_set == NULL)
     return FALSE;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_offset);
@@ -991,13 +1010,13 @@ gail_text_view_set_run_attributes (AtkEditableText *text,
 
   tag = gtk_text_buffer_create_tag (buffer, NULL, NULL);
 
-  for (index = 0; index < len; index++)
+  for (l = attrib_set; l; l = l->next)
     {
       gchar *name;
       gchar *value;
       AtkAttribute *at;
 
-      at = g_slist_nth (attrib_set, index)->data;
+      at = l->data;
 
       name = at->name;
       value = at->value;
@@ -1190,6 +1209,8 @@ gail_text_view_set_text_contents (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   /* The -1 indicates that the input string must be null-terminated */
@@ -1213,6 +1234,8 @@ gail_text_view_insert_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &pos_itr, *position);
@@ -1261,6 +1284,8 @@ gail_text_view_cut_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start, start_pos);
@@ -1287,6 +1312,8 @@ gail_text_view_delete_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   gtk_text_buffer_get_iter_at_offset (buffer, &start_itr, start_pos);
@@ -1309,6 +1336,8 @@ gail_text_view_paste_text (AtkEditableText *text,
     return;
 
   view = GTK_TEXT_VIEW (widget);
+  if (!gtk_text_view_get_editable (view))
+    return;
   buffer = view->buffer;
 
   paste_struct.buffer = buffer;
@@ -1400,9 +1429,11 @@ _gail_text_view_mark_set_cb (GtkTextBuffer *buffer,
 {
   GtkTextView *text = (GtkTextView *) user_data;
   AtkObject *accessible;
+  GailTextView *gail_text_view;
   const char *mark_name = gtk_text_mark_get_name(arg2);
 
   accessible = gtk_widget_get_accessible(GTK_WIDGET(text));
+  gail_text_view = GAIL_TEXT_VIEW (accessible);
 
   /*
    * Only generate the signal for the "insert" mark, which
@@ -1410,21 +1441,41 @@ _gail_text_view_mark_set_cb (GtkTextBuffer *buffer,
    */
   if (mark_name && !strcmp(mark_name, "insert"))
     {
-      GtkTextIter selection_bound_iter;
-      int insert_offset, selection_bound_offset;
+      int insert_offset, selection_bound;
+      gboolean selection_changed;
 
       insert_offset = gtk_text_iter_get_offset (arg1);
-      emit_text_caret_moved (accessible, insert_offset);
 
-      gtk_text_buffer_get_iter_at_mark (buffer, &selection_bound_iter,
-                 gtk_text_buffer_get_mark (buffer, "selection_bound"));
-      selection_bound_offset = gtk_text_iter_get_offset (&selection_bound_iter);
+      selection_bound = get_selection_bound (buffer);
+      if (selection_bound != insert_offset)
+        {
+          if (selection_bound != gail_text_view->previous_selection_bound ||
+              insert_offset != gail_text_view->previous_insert_offset)
+            {
+              selection_changed = TRUE;
+            }
+          else
+            {
+              selection_changed = FALSE;
+            }
+        }
+      else if (gail_text_view->previous_selection_bound != gail_text_view->previous_insert_offset)
+        {
+          selection_changed = TRUE;
+        }
+      else
+        {
+          selection_changed = FALSE;
+        }
+
+      emit_text_caret_moved (gail_text_view, insert_offset);
       /*
        * insert and selection_bound marks are different to a selection
        * has changed
        */
-      if (selection_bound_offset != insert_offset)
+      if (selection_changed)
         g_signal_emit_by_name (accessible, "text_selection_changed");
+      gail_text_view->previous_selection_bound = selection_bound;
     }
 }
 
@@ -1446,7 +1497,8 @@ _gail_text_view_changed_cb (GtkTextBuffer *buffer,
                              gail_text_view->length);
       gail_text_view->signal_name = NULL;
     }
-  emit_text_caret_moved (accessible, get_insert_offset (buffer));
+  emit_text_caret_moved (gail_text_view, get_insert_offset (buffer));
+  gail_text_view->previous_selection_bound = get_selection_bound (buffer);
 }
 
 static gchar*
@@ -1463,13 +1515,11 @@ get_text_near_offset (AtkText          *text,
   view = GTK_TEXT_VIEW (GTK_ACCESSIBLE (text)->widget);
 
   /*
-   * If wrapping is on for the GtkTextView, the GtkTextView is passed
-   * to the function gail_text_util_get_text() so it can find the start
-   * and end of the current line on the display.
+   * Pass the GtkTextView to the function gail_text_util_get_text() 
+   * so it can find the start and end of the current line on the display.
    */
-  if (view->wrap_mode != GTK_WRAP_NONE &&
-      (boundary_type == ATK_TEXT_BOUNDARY_LINE_START ||
-       boundary_type == ATK_TEXT_BOUNDARY_LINE_END))
+  if (boundary_type == ATK_TEXT_BOUNDARY_LINE_START ||
+      boundary_type == ATK_TEXT_BOUNDARY_LINE_END)
     layout = view;
 
   return gail_text_util_get_text (GAIL_TEXT_VIEW (text)->textutil, layout,
@@ -1488,14 +1538,21 @@ get_insert_offset (GtkTextBuffer *buffer)
   return gtk_text_iter_get_offset (&cursor_itr);
 }
 
-static void
-emit_text_caret_moved (AtkObject *obj,
-                       gint      insert_offset)
+static gint
+get_selection_bound (GtkTextBuffer *buffer)
 {
-  GailTextView *gail_text_view;
+  GtkTextMark *selection_mark;
+  GtkTextIter selection_itr;
 
-  gail_text_view = GAIL_TEXT_VIEW (obj);
+  selection_mark = gtk_text_buffer_get_selection_bound (buffer);
+  gtk_text_buffer_get_iter_at_mark (buffer, &selection_itr, selection_mark);
+  return gtk_text_iter_get_offset (&selection_itr);
+}
 
+static void
+emit_text_caret_moved (GailTextView *gail_text_view,
+                       gint          insert_offset)
+{
   if (insert_offset != gail_text_view->previous_insert_offset)
     {
       /*
@@ -1504,7 +1561,7 @@ emit_text_caret_moved (AtkObject *obj,
        * When mouse click is used to change caret position, notification
        * is received on button down and button up.
        */
-      g_signal_emit_by_name (obj, "text_caret_moved", insert_offset);
+      g_signal_emit_by_name (gail_text_view, "text_caret_moved", insert_offset);
       gail_text_view->previous_insert_offset = insert_offset;
     }
 }
