@@ -17,9 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
 #include <gtk/gtk.h>
 #include "gailtogglebutton.h"
-#include "gailtogglebuttonfactory.h"
 
 static void      gail_toggle_button_class_init        (GailToggleButtonClass *klass);
 
@@ -28,8 +28,8 @@ static void      gail_toggle_button_toggled_gtk       (GtkWidget             *wi
 static void      gail_toggle_button_real_notify_gtk   (GObject               *obj,
                                                        GParamSpec            *pspec);
 
-static void      gail_toggle_button_real_init         (GailWidget            *widget,
-                                                       GtkWidget             *gtk_widget);
+static void      gail_toggle_button_real_initialize   (AtkObject             *obj,
+                                                       gpointer              data);
 
 static AtkStateSet* gail_toggle_button_ref_state_set  (AtkObject             *accessible);
 
@@ -41,24 +41,24 @@ gail_toggle_button_get_type (void)
   static GType type = 0;
 
   if (!type)
-  {
-    static const GTypeInfo tinfo =
     {
-      sizeof (GailToggleButtonClass),
-      (GBaseInitFunc) NULL, /* base init */
-      (GBaseFinalizeFunc) NULL, /* base finalize */
-      (GClassInitFunc) gail_toggle_button_class_init, /* class init */
-      (GClassFinalizeFunc) NULL, /* class finalize */
-      NULL, /* class data */
-      sizeof (GailToggleButton), /* instance size */
-      0, /* nb preallocs */
-      (GInstanceInitFunc) NULL, /* instance init */
-      NULL /* value table */
-    };
+      static const GTypeInfo tinfo =
+      {
+        sizeof (GailToggleButtonClass),
+        (GBaseInitFunc) NULL, /* base init */
+        (GBaseFinalizeFunc) NULL, /* base finalize */
+        (GClassInitFunc) gail_toggle_button_class_init, /* class init */
+        (GClassFinalizeFunc) NULL, /* class finalize */
+        NULL, /* class data */
+        sizeof (GailToggleButton), /* instance size */
+        0, /* nb preallocs */
+        (GInstanceInitFunc) NULL, /* instance init */
+        NULL /* value table */
+      };
 
-    type = g_type_register_static (GAIL_TYPE_BUTTON,
-                                   "GailToggleButton", &tinfo, 0);
-  }
+      type = g_type_register_static (GAIL_TYPE_BUTTON,
+                                     "GailToggleButton", &tinfo, 0);
+    }
 
   return type;
 }
@@ -70,47 +70,42 @@ gail_toggle_button_class_init (GailToggleButtonClass *klass)
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
 
   widget_class = (GailWidgetClass*)klass;
-  widget_class->init = gail_toggle_button_real_init;
   widget_class->notify_gtk = gail_toggle_button_real_notify_gtk;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_BUTTON);
+  parent_class = g_type_class_peek_parent (klass);
 
   class->ref_state_set = gail_toggle_button_ref_state_set;
+  class->initialize = gail_toggle_button_real_initialize;
 }
 
-GtkAccessible* 
+AtkObject* 
 gail_toggle_button_new (GtkWidget *widget)
 {
   GObject *object;
-  GtkAccessible *accessible;
+  AtkObject *accessible;
 
   g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (widget), NULL);
 
   object = g_object_new (GAIL_TYPE_TOGGLE_BUTTON, NULL);
 
-  g_return_val_if_fail (GTK_IS_ACCESSIBLE (object), NULL);
+  accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
-  accessible = GTK_ACCESSIBLE (object);
   if (GTK_IS_CHECK_BUTTON (widget))
-  {
-    ATK_OBJECT(accessible)->role = ATK_ROLE_CHECK_BOX;
-  }
+    accessible->role = ATK_ROLE_CHECK_BOX;
   else
-  {
-    ATK_OBJECT(accessible)->role = ATK_ROLE_TOGGLE_BUTTON;
-  }
+    accessible->role = ATK_ROLE_TOGGLE_BUTTON;
+ 
   return accessible;
 }
 
 static void
-gail_toggle_button_real_init (GailWidget         *widget,
-                              GtkWidget          *gtk_widget)
+gail_toggle_button_real_initialize (AtkObject *obj,
+                                    gpointer  data)
 {
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  g_signal_connect (G_OBJECT (gtk_widget),
+  g_signal_connect (data,
                     "toggled",
                     G_CALLBACK (gail_toggle_button_toggled_gtk),
                     NULL);
@@ -144,7 +139,7 @@ gail_toggle_button_ref_state_set (AtkObject *accessible)
 
   toggle_button = GTK_TOGGLE_BUTTON (widget);
 
-  if (toggle_button->active)
+  if (gtk_toggle_button_get_active (toggle_button))
     atk_state_set_add_state (state_set, ATK_STATE_CHECKED);
 
   if (gtk_toggle_button_get_inconsistent (toggle_button))
@@ -166,8 +161,5 @@ gail_toggle_button_real_notify_gtk (GObject           *obj,
     atk_object_notify_state_change (atk_obj, ATK_STATE_ENABLED,
                        !gtk_toggle_button_get_inconsistent (toggle_button));
   else
-  {
     GAIL_WIDGET_CLASS (parent_class)->notify_gtk (obj, pspec);
-  }
 }
-
