@@ -206,6 +206,8 @@ gail_widget_real_initialize (AtkObject *obj,
                     "unmap",
                     G_CALLBACK (gail_widget_map_gtk),
                     NULL);
+  g_object_set_data (G_OBJECT (obj), "atk-component-layer",
+		     GINT_TO_POINTER (ATK_LAYER_WIDGET));
 }
 
 AtkObject* 
@@ -222,8 +224,6 @@ gail_widget_new (GtkWidget *widget)
   atk_object_initialize (accessible, widget);
 
   accessible->role = ATK_ROLE_UNKNOWN;
-  g_object_set_data (G_OBJECT (accessible), "atk-component-layer",
-		     GINT_TO_POINTER (ATK_LAYER_WIDGET));
 
   return accessible;
 }
@@ -457,7 +457,7 @@ gail_widget_get_index_in_parent (AtkObject *accessible)
     return -1;
   g_return_val_if_fail (GTK_IS_CONTAINER (parent_widget), -1);
 
-  children = gtk_container_children (GTK_CONTAINER (parent_widget));
+  children = gtk_container_get_children (GTK_CONTAINER (parent_widget));
 
   index = g_list_index (children, widget);
   g_list_free (children);
@@ -495,7 +495,7 @@ gail_widget_add_focus_handler (AtkComponent    *component,
   signal_id = g_signal_lookup ("focus-event", ATK_TYPE_OBJECT);
 
   ret = g_signal_handler_find (component, match_type, signal_id, 0, NULL,
-                               handler, NULL);
+                               (gpointer) handler, NULL);
   if (!ret)
     {
       return g_signal_connect_closure_by_id (component, 
