@@ -1206,6 +1206,9 @@ _print_text (AtkText *aobject)
       VALUE_STRING);
     g_free(output_str);
 
+    if (caret_offset < 0)
+      return(group_num);
+
     text_val = atk_text_get_text_at_offset (aobject, caret_offset,
                                             ATK_TEXT_BOUNDARY_CHAR,
                                             &start_offset, &end_offset);
